"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processPkgUrlCals2WeappData = exports.getIPAdress = exports.getMd5 = exports.getValidNodeModulesPath = exports.spawnPro = exports.isWinPlatform = exports.handlePathEscape = exports.remove = exports.getPackageJson = exports.compressingZip = exports.isLinux = exports.isMac = exports.isWindows = exports.promiseWrapper = exports.setReadOnly = void 0;
const pump_1 = __importDefault(require("pump"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const compressing_1 = __importDefault(require("compressing"));
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const crypto_1 = __importDefault(require("crypto"));
class ReadOnlyArray extends Array {
    constructor(mutable) {
        super();
        return Object.freeze(mutable.slice());
    }
}
function setReadOnly(target, key, v, deep = true) {
    let value = v;
    if (deep && typeof value === 'object') {
        if (Array.isArray(value)) {
            value = new ReadOnlyArray(value);
        }
        else {
            Object.entries(value).forEach(([k, v]) => setReadOnly(value, k, v));
        }
    }
    return Object.defineProperty(target, key, {
        writable: false,
        get() {
            return value;
        },
        set() {
            console.error(`Can not modify readonly value "${key}"`);
        },
    });
}
exports.setReadOnly = setReadOnly;
function promiseWrapper(p) {
    return new Promise((resolve) => {
        try {
            p.then((i) => resolve([null, i])).catch((e) => resolve([e, null]));
        }
        catch (e) {
            resolve([e, null]);
        }
    });
}
exports.promiseWrapper = promiseWrapper;
exports.isWindows = process.platform === 'win32';
exports.isMac = process.platform === 'darwin';
exports.isLinux = process.platform === 'linux';
function compressingZip(entryList, distPath, opts = {}) {
    return new Promise((resolve, reject) => {
        const zipStream = new compressing_1.default.zip.Stream();
        entryList.map((entry) => {
            zipStream.addEntry(entry, opts);
        });
        const destStream = fs_extra_1.default.createWriteStream(distPath);
        (0, pump_1.default)(zipStream, destStream, (err) => {
            if (err) {
                console.error(err);
                reject(err);
            }
            resolve(distPath);
        });
    });
}
exports.compressingZip = compressingZip;
function getPackageJson(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield fs_extra_1.default.readJson(path_1.default.join(cwd, 'package.json'));
    });
}
exports.getPackageJson = getPackageJson;
function remove(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs_extra_1.default.remove(dir);
    });
}
exports.remove = remove;
function handlePathEscape(path) {
    if (os_1.default.platform() !== 'win32') {
        return path;
    }
    return path.replace(/\\/g, '\\\\');
}
exports.handlePathEscape = handlePathEscape;
function isWinPlatform() {
    return os_1.default.platform() === 'win32';
}
exports.isWinPlatform = isWinPlatform;
function spawnPro(command, args, options) {
    return new Promise((resolve, reject) => {
        var _a, _b;
        const process = (0, cross_spawn_1.default)(command, args, options);
        let strOut = '';
        let strErr = '';
        (_a = process.stdout) === null || _a === void 0 ? void 0 : _a.on('data', (data) => {
            strOut += data;
        });
        (_b = process.stderr) === null || _b === void 0 ? void 0 : _b.on('data', (data) => {
            strErr += data;
        });
        function handleFinish(code) {
            if (code === 0) {
                resolve(strOut + strErr);
            }
            else {
                reject(strOut + strErr);
            }
        }
        process.on('error', (err) => {
            console.error('error', err);
        });
        process.on('close', handleFinish);
        process.on('exit', handleFinish);
    });
}
exports.spawnPro = spawnPro;
function getValidNodeModulesPath() {
    const nodeModulesPath = path_1.default.resolve(__dirname, '../../node_modules');
    const cwdWaPath = path_1.default.resolve(process.cwd(), 'node_modules/.bin/wa');
    if (fs_extra_1.default.pathExistsSync(cwdWaPath)) {
        return path_1.default.resolve(process.cwd(), 'node_modules');
    }
    return nodeModulesPath;
}
exports.getValidNodeModulesPath = getValidNodeModulesPath;
function getMd5(string, len = 8) {
    return crypto_1.default.createHash('md5').update(string).digest('hex').slice(0, len);
}
exports.getMd5 = getMd5;
function getIPAdress() {
    const interfaces = os_1.default.networkInterfaces();
    for (const devName in interfaces) {
        const iface = interfaces[devName] || [];
        for (let i = 0; i < iface.length; i++) {
            const alias = iface[i];
            if (alias.family === 'IPv4' &&
                alias.address !== '127.0.0.1' &&
                !alias.internal) {
                return alias.address;
            }
        }
    }
}
exports.getIPAdress = getIPAdress;
function processPkgUrlCals2WeappData(cals) {
    const { extra = {}, datasources, dataSources } = cals, restCals = __rest(cals, ["extra", "datasources", "dataSources"]);
    return Object.assign(Object.assign(Object.assign({}, extra), restCals), { datasources: datasources || dataSources || [] });
}
exports.processPkgUrlCals2WeappData = processPkgUrlCals2WeappData;
