"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HACK_FIX_LOWCODE_OUT = exports.HACK_FIX_LOWCODE_IN = exports.getExtByType = exports.originJsonToFileList = void 0;
function originJsonToFileList(appData) {
    const configFiles = handleConfigJson(appData);
    const pageFiles = handlePageJson(appData);
    const lowCodeFiles = handleLowCodeFile(appData);
    return [...configFiles, ...pageFiles, ...lowCodeFiles];
}
exports.originJsonToFileList = originJsonToFileList;
function handleConfigJson(appData) {
    const homePageItem = appData.pageInstanceList.find((page) => !!page.isHome);
    const globalConfig = {
        appConfig: appData.appConfig,
        selectedPageId: appData.selectedPageId,
        npmDependencies: appData.npmDependencies,
        homePage: homePageItem && homePageItem.id,
        themeVars: appData.themeVars,
        presetColors: appData.presetColors,
    };
    const globalFile = {
        path: 'global/app-config',
        name: 'app-config',
        type: 'json',
        code: JSON.stringify(globalConfig, null, 2),
        pageId: 'global',
    };
    const pagesConfig = appData.pageInstanceList.map((page) => {
        const pageConfig = {
            data: page.data,
            commonStyle: page.commonStyle,
        };
        return {
            path: `${page.id}/page-config`,
            name: 'page-config',
            type: 'json',
            code: JSON.stringify(pageConfig, null, 2),
            pageId: page.id,
        };
    });
    return [globalFile].concat(pagesConfig);
}
function handlePageJson(appData) {
    return appData.pageInstanceList.map((page) => {
        const pageJsonFile = {
            name: 'page',
            pageId: page.id,
            path: `${page.id}/page`,
            type: 'json',
            code: JSON.stringify(page.componentInstances || {}, null, 2),
        };
        return pageJsonFile;
    });
}
function handleLowCodeFile(appData) {
    const globalLowCodes = appData.lowCodes;
    const pagesLowCodes = appData.pageInstanceList.reduce((pageLowcode, page) => {
        return pageLowcode.concat(page.lowCodes);
    }, []);
    return [...globalLowCodes, ...pagesLowCodes];
}
function initOriginJson(fileList) {
    const data = {
        pageInstanceList: [],
        appConfig: {},
        lowCodes: getCommonLowCodes('global'),
        npmDependencies: {},
        plugins: [],
    };
    fileList
        .filter(({ pageId }) => pageId !== 'global')
        .forEach(({ pageId }) => {
        const pageInstance = data.pageInstanceList.find((page) => page.id === pageId);
        if (!pageInstance) {
            data.pageInstanceList.push({
                id: pageId,
                data: {},
                componentInstances: {},
                lowCodes: getCommonLowCodes(pageId),
                isHome: false,
                listeners: [],
                commonStyle: {},
                pluginInstances: [],
                styleBindPath: '',
            });
        }
    });
    return data;
}
function getCommonLowCodes(pageId) {
    const isGlobal = pageId === 'global';
    const commonLowCodes = [];
    if (isGlobal) {
        commonLowCodes.concat([
            {
                code: 'export default {}',
                name: '____index____',
                pageId: 'global',
                type: 'normal-module',
                path: 'global/common/____index____',
                system: false,
            },
        ]);
    }
    else {
        commonLowCodes.concat([
            {
                code: 'export default {}',
                name: '____index____',
                pageId,
                path: `${pageId}/handler/____index____`,
                type: 'handler-fn',
                system: false,
            },
        ]);
    }
    return commonLowCodes;
}
function getExtByType(type) {
    switch (type) {
        case 'app-style':
        case 'style':
        case 'theme':
            return '.less';
        case 'config':
        case 'json':
        case 'app-config':
        case 'page-config':
            return '.json';
        default:
            return '.js';
    }
}
exports.getExtByType = getExtByType;
function HACK_FIX_LOWCODE_IN(code = '') {
    return code.replace('../../common/', '../../../common/');
}
exports.HACK_FIX_LOWCODE_IN = HACK_FIX_LOWCODE_IN;
function HACK_FIX_LOWCODE_OUT(code = '') {
    return code.replace('../../../common/', '../../common/');
}
exports.HACK_FIX_LOWCODE_OUT = HACK_FIX_LOWCODE_OUT;
