import * as sdk from '@tcwd/weapps-sdk';
import {
  createWebApp,
  _ACTIONS_KEY as ACTIONS_KEY,
  _ROUTER_KEY as ROUTER_KEY,
} from '@cloudbase/weda-client';
import { createComputed } from 'utils';
import { createDataset } from '../datasources';
import store, { subPackageName } from '../store';
import computed from '../store/computed';
import common from './common';
import { formatEnum, enumOptions } from '../utils/formatEnum';
import { _isMobile } from '../utils';
import actionMap from './material-actions';
import { get as lodashGet } from 'lodash';

const mainAppKey = '__weappsMainApp';
const appGlobal = process.env.isMiniprogram ? getApp() : window;

export const app = createGlboalApi();
export const $page = createPageApi();

function createGlboalApi() {
  const webApp = createWebApp({
    id: '<%= appId %>',
    domain: '<%= domain %>',
    appConfig: <%= appConfig %>,
  });
  const globalAPI = {
    id: '<%= appId %>',
    domain: '<%= domain %>',
    formActions: {},
    pages: {},
    session: {
      // configure: sdk.configure,
      // request: sdk.request,
      // getSessionId: sdk.getSessionId,
    },
    state: store,
    computed: createComputed(computed.global),
    common,
    invoke(params) {
      return $page.invokeComponentMethod(params);
    },
    ...webApp,
  };

  globalAPI.__internal__.enumOptions = enumOptions;
  globalAPI.utils.formatEnum = formatEnum;
  globalAPI.utils._getConfig = () => {
    console.warn('deprecated app.utils._getConfig');
    return globalAPI.__internal__.getConfig();
  }

  const dataset = createDataset('$global');
  globalAPI.dataset = dataset;
  globalAPI.state.dataset = dataset;
  globalAPI.setState = (userSetState) => {
    Object.keys(userSetState).forEach((keyPath) => {
      globalAPI.utils.set(globalAPI.dataset.state, keyPath, userSetState[keyPath]);
    });
  };
  if (subPackageName) {
    // is sub app
    globalAPI.mainApp = appGlobal[mainAppKey];
  } else {
    // is mainApp
    appGlobal.app = globalAPI;
    appGlobal[mainAppKey] = globalAPI;
  }

  // 挂运营平台上报对象到app里
  globalAPI.yyptReport = appGlobal.yyptReport;

  // # expose some sdk modules
  const sdkModsIncluded = ['flow', 'getPageOptions', 'getLaunchOptions'];
  sdkModsIncluded.forEach((key) => {
    globalAPI[key] = sdk[key];
  });
  return globalAPI;
}
function createPageApi() {
  const $page = {
    state: {},
    computed: {},
    handler: {},
    props: {},
    widgets: {},
    // 页面数据源变量存储位置
    dataVar: {},
  };
  $page.invokeComponentMethod = ({ component, method, params }) => {
    const widgetData = lodashGet($page.widgets, component);
    if (Array.isArray(widgetData)) {
      throw new Error('调用方法失败：id为' + component + '的组件拥有多个实例');
      // widgetData.forEach(widget => {
      //   if(!widget._methods[method]) {
      //     throw new Error('NOT_FOUND')
      //   } else {
      //     actions.push(widget._methods[method])
      //   }
      // })
    } else {
      if (!widgetData._methods[method] || typeof widgetData._methods[method] != 'function') {
        throw new Error('调用方法失败：未找到id为' + component + '下的方法' + method);
      }
      return widgetData._methods[method](params);
    }
  };
  return $page;
}

// 分app 和 wx 挂载app
export const mountAPIs = (sdks) => {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];
    if(item === 'showModal') {
      const OFFICIAL_COMPONENT_LIB = 'gsd-h5-react';
      const showModal = actionMap[OFFICIAL_COMPONENT_LIB]?.showModal;
      if (!_isMobile() && showModal) {
        action = function (params) {
          return showModal({ data: params });
        };
      }
      app[item] = action;
    }
    else if (!(item in ACTIONS_KEY) && !(item in ROUTER_KEY)) {
      app[item] = action;
    }
  });
  return app;
};
